import * as anchor from '@project-serum/anchor';
import { Program } from '@project-serum/anchor';
import { MetadataParams, NetworkType, RepayDebtVoucherInformationParams, VoucherNftType } from './types';
import { PDA } from './pda';
import { Keypair, PublicKey } from '@solana/web3.js';
export declare class VoucherNftFixture {
    readonly program: Program<VoucherNftType>;
    readonly connection: anchor.web3.Connection;
    readonly provider: anchor.AnchorProvider;
    readonly programId: anchor.web3.PublicKey;
    readonly pda: PDA;
    private readonly verbose;
    constructor(keypair: anchor.web3.Keypair, connectionUrl: string, programId: string, verbose?: boolean);
    initialize(relendMint: Keypair, collectionParams: MetadataParams): Promise<string>;
    addVault(seed: string, operator: PublicKey): Promise<string>;
    mintVoucher(seed: string, operator: Keypair, mint: Keypair, params: MetadataParams): Promise<string>;
    mintVoucherRepay(seed: string, operator: Keypair, mint: Keypair, metadataParams: MetadataParams, repayDebtVoucherInformationParams: RepayDebtVoucherInformationParams): Promise<string>;
    sendVoucher(seed: string, operator: Keypair | PublicKey, mint: PublicKey, user: PublicKey): Promise<string>;
    sendVouchers(seed: string, operator: Keypair, mints: PublicKey[], users: PublicKey[]): Promise<string>;
    getConfigData(): Promise<import("@project-serum/anchor/dist/cjs/program/namespace/types").TypeDef<{
        name: "authority";
        type: {
            "kind": "struct";
            "fields": [{
                "name": "bump";
                "type": "u8";
            }];
        };
    } | {
        name: "config";
        type: {
            "kind": "struct";
            "fields": [{
                "name": "admin";
                "type": "publicKey";
            }, {
                "name": "collection";
                "type": "publicKey";
            }, {
                "name": "reserve";
                "type": {
                    "array": ["u128", 6];
                };
            }];
        };
    } | {
        name: "vault";
        type: {
            "kind": "struct";
            "fields": [{
                "name": "admin";
                "type": "publicKey";
            }, {
                "name": "operator";
                "type": "publicKey";
            }, {
                "name": "bump";
                "type": "u8";
            }, {
                "name": "seed";
                "type": "string";
            }, {
                "name": "reserve";
                "type": {
                    "array": ["u128", 6];
                };
            }];
        };
    } | {
        name: "repayDebtVoucher";
        type: {
            "kind": "struct";
            "fields": [{
                "name": "discountPercentage";
                "type": "u16";
            }, {
                "name": "maximumAmount";
                "type": "u32";
            }, {
                "name": "startTime";
                "type": "i64";
            }, {
                "name": "endTime";
                "type": "i64";
            }, {
                "name": "nftMint";
                "type": "publicKey";
            }, {
                "name": "authority";
                "type": "publicKey";
            }, {
                "name": "reserve";
                "type": {
                    "array": ["u128", 6];
                };
            }, {
                "name": "eligibleTokens";
                "type": {
                    "vec": "publicKey";
                };
            }];
        };
    }, anchor.IdlTypes<import("./artifacts/voucher_nft").VoucherNft>>>;
    getAuthorityData(): Promise<import("@project-serum/anchor/dist/cjs/program/namespace/types").TypeDef<{
        name: "authority";
        type: {
            "kind": "struct";
            "fields": [{
                "name": "bump";
                "type": "u8";
            }];
        };
    } | {
        name: "config";
        type: {
            "kind": "struct";
            "fields": [{
                "name": "admin";
                "type": "publicKey";
            }, {
                "name": "collection";
                "type": "publicKey";
            }, {
                "name": "reserve";
                "type": {
                    "array": ["u128", 6];
                };
            }];
        };
    } | {
        name: "vault";
        type: {
            "kind": "struct";
            "fields": [{
                "name": "admin";
                "type": "publicKey";
            }, {
                "name": "operator";
                "type": "publicKey";
            }, {
                "name": "bump";
                "type": "u8";
            }, {
                "name": "seed";
                "type": "string";
            }, {
                "name": "reserve";
                "type": {
                    "array": ["u128", 6];
                };
            }];
        };
    } | {
        name: "repayDebtVoucher";
        type: {
            "kind": "struct";
            "fields": [{
                "name": "discountPercentage";
                "type": "u16";
            }, {
                "name": "maximumAmount";
                "type": "u32";
            }, {
                "name": "startTime";
                "type": "i64";
            }, {
                "name": "endTime";
                "type": "i64";
            }, {
                "name": "nftMint";
                "type": "publicKey";
            }, {
                "name": "authority";
                "type": "publicKey";
            }, {
                "name": "reserve";
                "type": {
                    "array": ["u128", 6];
                };
            }, {
                "name": "eligibleTokens";
                "type": {
                    "vec": "publicKey";
                };
            }];
        };
    }, anchor.IdlTypes<import("./artifacts/voucher_nft").VoucherNft>>>;
    getVaultData(seed: string): Promise<import("@project-serum/anchor/dist/cjs/program/namespace/types").TypeDef<{
        name: "authority";
        type: {
            "kind": "struct";
            "fields": [{
                "name": "bump";
                "type": "u8";
            }];
        };
    } | {
        name: "config";
        type: {
            "kind": "struct";
            "fields": [{
                "name": "admin";
                "type": "publicKey";
            }, {
                "name": "collection";
                "type": "publicKey";
            }, {
                "name": "reserve";
                "type": {
                    "array": ["u128", 6];
                };
            }];
        };
    } | {
        name: "vault";
        type: {
            "kind": "struct";
            "fields": [{
                "name": "admin";
                "type": "publicKey";
            }, {
                "name": "operator";
                "type": "publicKey";
            }, {
                "name": "bump";
                "type": "u8";
            }, {
                "name": "seed";
                "type": "string";
            }, {
                "name": "reserve";
                "type": {
                    "array": ["u128", 6];
                };
            }];
        };
    } | {
        name: "repayDebtVoucher";
        type: {
            "kind": "struct";
            "fields": [{
                "name": "discountPercentage";
                "type": "u16";
            }, {
                "name": "maximumAmount";
                "type": "u32";
            }, {
                "name": "startTime";
                "type": "i64";
            }, {
                "name": "endTime";
                "type": "i64";
            }, {
                "name": "nftMint";
                "type": "publicKey";
            }, {
                "name": "authority";
                "type": "publicKey";
            }, {
                "name": "reserve";
                "type": {
                    "array": ["u128", 6];
                };
            }, {
                "name": "eligibleTokens";
                "type": {
                    "vec": "publicKey";
                };
            }];
        };
    }, anchor.IdlTypes<import("./artifacts/voucher_nft").VoucherNft>>>;
    getRepayDebtVoucherData(mint: PublicKey): Promise<import("@project-serum/anchor/dist/cjs/program/namespace/types").TypeDef<{
        name: "authority";
        type: {
            "kind": "struct";
            "fields": [{
                "name": "bump";
                "type": "u8";
            }];
        };
    } | {
        name: "config";
        type: {
            "kind": "struct";
            "fields": [{
                "name": "admin";
                "type": "publicKey";
            }, {
                "name": "collection";
                "type": "publicKey";
            }, {
                "name": "reserve";
                "type": {
                    "array": ["u128", 6];
                };
            }];
        };
    } | {
        name: "vault";
        type: {
            "kind": "struct";
            "fields": [{
                "name": "admin";
                "type": "publicKey";
            }, {
                "name": "operator";
                "type": "publicKey";
            }, {
                "name": "bump";
                "type": "u8";
            }, {
                "name": "seed";
                "type": "string";
            }, {
                "name": "reserve";
                "type": {
                    "array": ["u128", 6];
                };
            }];
        };
    } | {
        name: "repayDebtVoucher";
        type: {
            "kind": "struct";
            "fields": [{
                "name": "discountPercentage";
                "type": "u16";
            }, {
                "name": "maximumAmount";
                "type": "u32";
            }, {
                "name": "startTime";
                "type": "i64";
            }, {
                "name": "endTime";
                "type": "i64";
            }, {
                "name": "nftMint";
                "type": "publicKey";
            }, {
                "name": "authority";
                "type": "publicKey";
            }, {
                "name": "reserve";
                "type": {
                    "array": ["u128", 6];
                };
            }, {
                "name": "eligibleTokens";
                "type": {
                    "vec": "publicKey";
                };
            }];
        };
    }, anchor.IdlTypes<import("./artifacts/voucher_nft").VoucherNft>>>;
}
export declare class VoucherNftFixtureBuilder {
    private keypairPath;
    private programId;
    private networkType;
    private verbose;
    constructor();
    withNetwork(network: NetworkType): this;
    withKeypair(keypairPath: string): this;
    withProgramId(programId: string): this;
    withVerbose(verbose: boolean): this;
    build(): Promise<VoucherNftFixture>;
}
