"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.VoucherNftFixtureBuilder = exports.VoucherNftFixture = void 0;
const anchor = __importStar(require("@project-serum/anchor"));
const token = __importStar(require("@solana/spl-token"));
const anchor_1 = require("@project-serum/anchor");
const types_1 = require("./types.js");
const helpers_1 = require("@solana-developers/helpers");
const pda_1 = require("./pda.js");
const instructions_1 = require("./instructions.js");
const constants_1 = require("./constants.js");
class VoucherNftFixture {
    program;
    connection;
    provider;
    programId;
    pda;
    verbose;
    constructor(keypair, connectionUrl, programId, verbose = false) {
        this.connection = new anchor.web3.Connection(connectionUrl, { commitment: 'confirmed' });
        this.provider = new anchor.AnchorProvider(this.connection, new anchor.Wallet(keypair), {
            commitment: 'confirmed',
        });
        this.programId = new anchor.web3.PublicKey(programId);
        this.pda = new pda_1.PDA(this.programId);
        this.program = new anchor_1.Program(types_1.VoucherNftIDL, this.programId, this.provider);
        this.verbose = verbose;
    }
    async initialize(relendMint, collectionParams) {
        const { key: config } = this.pda.config();
        const { key: authority } = this.pda.authority();
        const { key: metadata } = await this.pda.metadata(relendMint.publicKey);
        const { key: masterEdition } = await this.pda.masterEdition(relendMint.publicKey);
        const authorityTokenAccount = await token.getAssociatedTokenAddress(relendMint.publicKey, authority, true);
        try {
            return await this.program.methods
                .initialize(collectionParams)
                .accounts({
                config,
                authority: authority,
                admin: this.provider.publicKey,
                relendCollection: relendMint.publicKey,
                metadataAccount: metadata,
                authorityTokenAccount,
                masterEdition,
                tokenMetadataProgram: constants_1.Constants.TOKEN_METADATA_PROGRAM,
            })
                .signers([relendMint])
                .rpc();
        }
        catch (error) {
            this.verbose && console.error(error);
            throw error;
        }
    }
    async addVault(seed, operator) {
        try {
            const { key: config } = this.pda.config();
            const { key: vault } = this.pda.vault(seed);
            const addVaultIns = await (0, instructions_1.addVaultIx)(this.program, {
                admin: this.provider.publicKey,
                config,
                operator: operator,
                seed: seed,
                vault: vault,
            });
            const transaction = new anchor.web3.Transaction().add(addVaultIns);
            return await this.provider.sendAndConfirm(transaction);
        }
        catch (error) {
            this.verbose && console.error(error);
            throw error;
        }
    }
    async mintVoucher(seed, operator, mint, params) {
        try {
            const { collection: collectionMint } = await this.getConfigData();
            const { key: collectionMetadata } = await this.pda.metadata(collectionMint);
            const { key: collectionMasterEdition } = await this.pda.masterEdition(collectionMint);
            const { key: metadataAccount } = await this.pda.metadata(mint.publicKey);
            const { key: masterEdition } = await this.pda.masterEdition(mint.publicKey);
            const { key: authority } = this.pda.authority();
            const { key: vault } = this.pda.vault(seed);
            const { key: config } = this.pda.config();
            const vaultTokenAccount = await token.getAssociatedTokenAddress(mint.publicKey, vault, true);
            const modifyUnitIns = (0, instructions_1.modifyComputeUnitIx)();
            const mintVoucherIns = await (0, instructions_1.mintVoucherIx)(this.program, {
                config,
                operator: operator.publicKey,
                authority: authority,
                tokenMetadataProgram: constants_1.Constants.TOKEN_METADATA_PROGRAM,
                vaultTokenAccount,
                metadataAccount,
                masterEdition,
                vault: vault,
                mint,
                collection: collectionMint,
                collectionMetadata,
                collectionMasterEdition,
                params,
            });
            const transaction = new anchor.web3.Transaction().add(modifyUnitIns, mintVoucherIns);
            return await this.provider.sendAndConfirm(transaction, [operator, mint]);
        }
        catch (error) {
            this.verbose && console.error(error);
            throw error;
        }
    }
    async mintVoucherRepay(seed, operator, mint, metadataParams, repayDebtVoucherInformationParams) {
        try {
            const { collection: collectionMint } = await this.getConfigData();
            const { key: collectionMetadata } = await this.pda.metadata(collectionMint);
            const { key: collectionMasterEdition } = await this.pda.masterEdition(collectionMint);
            const { key: metadataAccount } = await this.pda.metadata(mint.publicKey);
            const { key: masterEdition } = await this.pda.masterEdition(mint.publicKey);
            const { key: authority } = this.pda.authority();
            const { key: vault } = this.pda.vault(seed);
            const { key: config } = this.pda.config();
            const { key: repayDebtVoucher } = this.pda.repayDebtVoucher(mint.publicKey);
            const vaultTokenAccount = await token.getAssociatedTokenAddress(mint.publicKey, vault, true);
            const modifyUnitIns = (0, instructions_1.modifyComputeUnitIx)();
            const mintVoucherIns = await (0, instructions_1.mintVoucherIx)(this.program, {
                config,
                operator: operator.publicKey,
                authority: authority,
                tokenMetadataProgram: constants_1.Constants.TOKEN_METADATA_PROGRAM,
                vaultTokenAccount,
                metadataAccount,
                masterEdition,
                vault,
                mint,
                collection: collectionMint,
                collectionMetadata,
                collectionMasterEdition,
                params: metadataParams,
            });
            const addRepayDebtVoucherIns = await (0, instructions_1.addRepayDebtVoucherIx)(this.program, {
                authority: authority,
                masterEdition,
                metadataAccount,
                mint,
                operator: operator.publicKey,
                tokenMetadataProgram: constants_1.Constants.TOKEN_METADATA_PROGRAM,
                vault,
                repayDebtVoucher,
                params: repayDebtVoucherInformationParams,
            });
            const transaction = new anchor.web3.Transaction().add(modifyUnitIns, mintVoucherIns, addRepayDebtVoucherIns);
            return await this.provider.sendAndConfirm(transaction, [operator, mint]);
        }
        catch (error) {
            this.verbose && console.error(error);
            throw error;
        }
    }
    async sendVoucher(seed, operator, mint, user) {
        try {
            const { key: vault } = this.pda.vault(seed);
            const { key: masterEdition } = await this.pda.masterEdition(mint);
            const userTokenAccount = await token.getAssociatedTokenAddress(mint, user, false);
            const vaultTokenAccount = await token.getAssociatedTokenAddress(mint, vault, true);
            const sendVoucherIns = await (0, instructions_1.sendVoucherToUserIx)(this.program, {
                mint: mint,
                masterEdition,
                operator: operator instanceof anchor.web3.Keypair ? operator.publicKey : operator,
                user,
                userTokenAccount: userTokenAccount,
                vault: vault,
                vaultTokenAccount: vaultTokenAccount,
            });
            const transaction = new anchor.web3.Transaction().add(sendVoucherIns);
            return await this.provider.sendAndConfirm(transaction, operator instanceof anchor.web3.Keypair ? [operator] : []);
        }
        catch (error) {
            this.verbose && console.error(error);
            throw error;
        }
    }
    async sendVouchers(seed, operator, mints, users) {
        const { key: vault } = this.pda.vault(seed);
        if (mints.length !== users.length) {
            throw new Error('mints and users length mismatch');
        }
        const transactions = [];
        const transaction = new anchor.web3.Transaction();
        for (let i = 0; i < mints.length; i++) {
            const mint = mints[i];
            const user = users[i];
            const { key: masterEdition } = await this.pda.masterEdition(mint);
            const vaultTokenAccount = await token.getAssociatedTokenAddress(mint, vault, true);
            const userTokenAccount = await token.getAssociatedTokenAddress(mint, user, false);
            const sendVoucherIns = await (0, instructions_1.sendVoucherToUserIx)(this.program, {
                mint: mint,
                masterEdition,
                operator: operator.publicKey,
                user,
                userTokenAccount: userTokenAccount,
                vault: vault,
                vaultTokenAccount: vaultTokenAccount,
            });
            transaction.add(sendVoucherIns);
        }
        try {
            return await this.provider.sendAndConfirm(transaction, [operator]);
        }
        catch (error) {
            this.verbose && console.error(error);
            throw error;
        }
    }
    async getConfigData() {
        const { key: config } = this.pda.config();
        try {
            return await this.program.account.config.fetch(config);
        }
        catch (error) {
            this.verbose && console.error(error);
            throw error;
        }
    }
    async getAuthorityData() {
        const { key: authority } = this.pda.authority();
        try {
            return await this.program.account.authority.fetch(authority);
        }
        catch (error) {
            this.verbose && console.error(error);
            throw error;
        }
    }
    async getVaultData(seed) {
        const { key: vault } = this.pda.vault(seed);
        try {
            return await this.program.account.vault.fetch(vault);
        }
        catch (error) {
            this.verbose && console.error(error);
            throw error;
        }
    }
    async getRepayDebtVoucherData(mint) {
        const { key: repayDebtVoucher } = this.pda.repayDebtVoucher(mint);
        try {
            return await this.program.account.repayDebtVoucher.fetch(repayDebtVoucher);
        }
        catch (error) {
            this.verbose && console.error(error);
            throw error;
        }
    }
}
exports.VoucherNftFixture = VoucherNftFixture;
class VoucherNftFixtureBuilder {
    keypairPath;
    programId;
    networkType;
    verbose;
    constructor() {
        this.keypairPath = "";
        this.programId = "";
        this.networkType = types_1.NetworkType.LocalNet;
        this.verbose = true;
    }
    withNetwork(network) {
        this.networkType = network;
        return this;
    }
    withKeypair(keypairPath) {
        this.keypairPath = keypairPath;
        return this;
    }
    withProgramId(programId) {
        this.programId = programId;
        return this;
    }
    withVerbose(verbose) {
        this.verbose = verbose;
        return this;
    }
    async build() {
        const keypair = await (0, helpers_1.getKeypairFromFile)(this.keypairPath || types_1.configurations.get(this.networkType)?.keypairPath || "");
        const connectionUrl = types_1.configurations.get(this.networkType)?.url || "";
        const programId = this.programId || types_1.configurations.get(this.networkType)?.programId || "";
        return new VoucherNftFixture(keypair, connectionUrl, programId, this.verbose);
    }
}
exports.VoucherNftFixtureBuilder = VoucherNftFixtureBuilder;
